@@ -2,8 +2,7 @@
 
 extern crate libc;
 
-use std::ffi;
-use std::mem;
+use std::ffi::CStr;
 use std::ops::Deref;
 use std::ptr;
 use std::slice;
@@ -19,7 +18,7 @@ pub struct MallocBuffer<T> {
 impl<T> MallocBuffer<T> {
     /// Constructs a new `MallocBuffer` for a `malloc`'d buffer
     /// with the given length at the given pointer.
-    /// Returns `None` if the given pointer is null.
+    /// Returns `None` if the given pointer is null and the length is not 0.
     ///
     /// When this `MallocBuffer` drops, the elements of the buffer will be
     /// dropped and the buffer will be `free`'d.
@@ -27,7 +26,7 @@ impl<T> MallocBuffer<T> {
     /// Unsafe because there must be `len` contiguous, valid instances of `T`
     /// at `ptr`.
     pub unsafe fn new(ptr: *mut T, len: usize) -> Option<MallocBuffer<T>> {
-        if ptr.is_null() {
+        if len > 0 && ptr.is_null() {
             None
         } else {
             Some(MallocBuffer { ptr: ptr, len: len })
@@ -51,10 +50,14 @@ impl<T> Deref for MallocBuffer<T> {
     type Target = [T];
 
     fn deref(&self) -> &[T] {
-        let const_ptr = self.ptr as *const T;
+        let ptr = if self.len == 0 && self.ptr.is_null() {
+            // Even a 0-size slice cannot be null, so just use another pointer
+            0x1 as *const T
+        } else {
+            self.ptr as *const T
+        };
         unsafe {
-            let s = slice::from_raw_buf(&const_ptr, self.len);
-            mem::transmute(s)
+            slice::from_raw_parts(ptr, self.len)
         }
     }
 }
@@ -74,8 +77,8 @@ impl MallocString {
         if ptr.is_null() {
             None
         } else {
-            let const_ptr = ptr as *const c_char;
-            let bytes = ffi::c_str_to_bytes(&const_ptr);
+            let s = CStr::from_ptr(ptr as *const c_char);
+            let bytes = s.to_bytes();
             if str::from_utf8(bytes).is_ok() {
                 let data = MallocBuffer {
                     ptr: ptr as *mut u8,
@@ -100,3 +103,50 @@ impl Deref for MallocString {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use std::ptr;
+    use libc::{c_char, self};
+
+    use super::{MallocBuffer, MallocString};
+
+    #[test]
+    fn test_null_buf() {
+        let buf = unsafe {
+            MallocBuffer::<u32>::new(ptr::null_mut(), 0).unwrap()
+        };
+        assert!(&*buf == []);
+        assert!(Some(&*buf) == Some(&[]));
+
+        let buf = unsafe {
+            MallocBuffer::<u32>::new(ptr::null_mut(), 7)
+        };
+        assert!(buf.is_none());
+    }
+
+    #[test]
+    fn test_buf() {
+        let buf = unsafe {
+            let ptr = libc::malloc(12) as *mut u32;
+            *ptr = 1;
+            *ptr.offset(1) = 2;
+            *ptr.offset(2) = 3;
+            MallocBuffer::new(ptr, 3).unwrap()
+        };
+        assert!(&*buf == [1, 2, 3]);
+    }
+
+    #[test]
+    fn test_string() {
+        let s = unsafe {
+            let ptr = libc::malloc(4) as *mut c_char;
+            *ptr = 'h' as c_char;
+            *ptr.offset(1) = 'e' as c_char;
+            *ptr.offset(2) = 'y' as c_char;
+            *ptr.offset(3) = '\0' as c_char;
+            MallocString::new(ptr).unwrap()
+        };
+        assert!(&*s == "hey");
+    }
+}
